@@ -139,6 +139,8 @@ typedef char *(*FMT_ANGLE_FUNC) (char *buf, double theta, SRC_LINE line);
 FMT_ANGLE_FUNC fmt_angle_tbl[] = {
   fmt_angle_pst,
   fmt_angle_tikz,
+  fmt_angle_pst,
+  fmt_angle_tikz,
 };
 
 // this parses, substitues, notes any errors, and (if f is set) prints special output
@@ -307,11 +309,24 @@ emit_points_tkz (FILE * f, POINT_LIST_3D * pts, char *twixt, char *final)
 static void
 emit_dots_tkz (FILE * f, OBJECT * obj)
 {
+  static char *skip[] = { "dotsize", NULL };
+  char *dotsize, *cmd;
+
   DOTS_OBJECT *dots = (DOTS_OBJECT *) obj;
-  fprintf (f, "\\fill");
-  emit_opts (f, dots->opts, global_env->output_language);
-  emit_points_tkz (f, dots->pts, " circle (2pt)", " circle (2pt)");
+
+  // An ugly hack because TikZ uses special syntax for circles...
+  dotsize = opt_val(dots->opts, "dotsize");
+  if (dotsize == NULL)
+    dotsize = "2pt";
+  cmd = safe_malloc(strlen(dotsize) + 100);
+  sprintf(cmd, " circle (%s)", dotsize);
+
+  fprintf (f, "\\filldraw");
+  emit_opts_with_exceptions (f, dots->opts, skip, global_env->output_language);
+  emit_points_tkz (f, dots->pts, cmd, cmd);
   fprintf (f, ";\n");
+
+  safe_free(cmd);
 }
 
 static void
@@ -371,12 +386,14 @@ static EMIT_FUNC emit_tbl_tkz[] = {
 static EMIT_FUNC *emit_tbl_tbl[] = {
   emit_tbl_pst,
   emit_tbl_tkz,
+  emit_tbl_pst,
+  emit_tbl_tkz,
 };
 
 #define DOC_TEMPLATE_ESCAPE_STRING "%%SKETCH_OUTPUT%%"
 #define DOC_TEMPLATE_ESCAPE_STRING_LEN (sizeof(DOC_TEMPLATE_ESCAPE_STRING) - 1)
 
-char standard_us_doc_template_tikz[] =
+char standard_us_doc_template_tikz_latex[] =
   "\\documentclass[letterpaper,12pt]{article}\n"
   "\\usepackage[x11names,rgb]{xcolor}\n"
   "\\usepackage{tikz}\n"
@@ -399,7 +416,7 @@ char standard_us_doc_template_tikz[] =
   DOC_TEMPLATE_ESCAPE_STRING "\n"
   "\\end{center}\n" "\\vspace*{\\fill}\n" "\\end{document}\n";
 
-char standard_euro_doc_template_tikz[] =
+char standard_euro_doc_template_tikz_latex[] =
   "\\documentclass[a4paper,12pt]{article}\n"
   "\\usepackage[x11names,rgb]{xcolor}\n"
   "\\usepackage{tikz}\n"
@@ -422,10 +439,11 @@ char standard_euro_doc_template_tikz[] =
   DOC_TEMPLATE_ESCAPE_STRING "\n"
   "\\end{center}\n" "\\vspace*{\\fill}\n" "\\end{document}\n";
 
-char standard_us_doc_template_pst[] =
+char standard_us_doc_template_pst_latex[] =
   "\\documentclass[letterpaper,12pt]{article}\n"
   "\\usepackage{amsmath}\n"
   "\\usepackage{pstricks}\n"
+  "\\usepackage{pstricks-add}\n"
   "\\oddsidemargin 0in\n"
   "\\evensidemargin 0in\n"
   "\\topmargin 0in\n"
@@ -440,10 +458,11 @@ char standard_us_doc_template_pst[] =
   DOC_TEMPLATE_ESCAPE_STRING "\n"
   "\\end{center}\n" "\\vspace*{\\fill}\n" "\\end{document}\n";
 
-char standard_euro_doc_template_pst[] =
+char standard_euro_doc_template_pst_latex[] =
   "\\documentclass[a4paper,12pt]{article}\n"
   "\\usepackage{amsmath}\n"
   "\\usepackage{pstricks}\n"
+  "\\usepackage{pstricks-add}\n"
   "\\oddsidemargin -10mm\n"
   "\\evensidemargin -10mm\n"
   "\\topmargin 5mm\n"
@@ -458,14 +477,48 @@ char standard_euro_doc_template_pst[] =
   DOC_TEMPLATE_ESCAPE_STRING "\n"
   "\\end{center}\n" "\\vspace*{\\fill}\n" "\\end{document}\n";
 
+/* ---------------------------------------------------------------------- */
+
+char standard_us_doc_template_tikz_context[] =
+  "\\usemodule[tikz] \\usetikzlibrary[snakes,arrows,shapes,backgrounds]\n"
+  "\\setuppapersize[letter][letter]\n"
+  "\\setuplayout[topspace=0in,backspace=0in,header=0in,footer=0in,height=middle,width=middle]\n"
+  "\\setuppagenumbering[state=stop] % no page numbers\n"
+  "\\starttext\n"
+  "\\startalignment[middle]\n"
+  DOC_TEMPLATE_ESCAPE_STRING "\n"
+  "\\stopalignment\n"
+  "\\stoptext\n";
+
+char standard_euro_doc_template_tikz_context[] =
+  "\\usemodule[tikz] \\usetikzlibrary[snakes,arrows,shapes,backgrounds]\n"
+  "\\setuppapersize[a4][a4]\n"
+  "\\setuplayout[topspace=0cm,backspace=0cm,header=0cm,footer=0cm,height=middle,width=middle]\n"
+  "\\setuppagenumbering[state=stop] % no page numbers\n"
+  "\\starttext\n"
+  "\\startalignment[middle]\n"
+  DOC_TEMPLATE_ESCAPE_STRING "\n"
+  "\\stopalignment\n"
+  "\\stoptext\n";
+
+char standard_us_doc_template_pst_context[] =
+  "PSTricks does not work with ConTeXt as of 1 Feb 2008.\n";
+
+char standard_euro_doc_template_pst_context[] =
+  "PSTricks does not work with ConTeXt as of 1 Feb 2008.\n";
+
 char *standard_us_doc_template[] = {
-  standard_us_doc_template_pst,
-  standard_us_doc_template_tikz,
+  standard_us_doc_template_pst_latex,
+  standard_us_doc_template_tikz_latex,
+  standard_us_doc_template_pst_context,
+  standard_us_doc_template_tikz_context,
 };
 
 char *standard_euro_doc_template[] = {
-  standard_euro_doc_template_pst,
-  standard_euro_doc_template_tikz,
+  standard_euro_doc_template_pst_latex,
+  standard_euro_doc_template_tikz_latex,
+  standard_euro_doc_template_pst_context,
+  standard_euro_doc_template_tikz_context,
 };
 
 char *
@@ -505,7 +558,7 @@ doc_template_from_file (char *file_name, int output_language)
     {
       err (no_line, "can't open document template '%s%' for input\n",
 	   file_name);
-      return safe_strdup (standard_us_doc_template_pst);
+      return safe_strdup (standard_us_doc_template_pst_latex);
     }
   r = read_file_as_string (f);
   fclose (f);
@@ -513,7 +566,7 @@ doc_template_from_file (char *file_name, int output_language)
 }
 
 void
-emit_preamble_pst (FILE * f, BOX_3D * ext, GLOBAL_ENV * env)
+emit_preamble_pst_latex (FILE * f, BOX_3D * ext, GLOBAL_ENV * env)
 {
   char buf1[16], buf2[16], buf3[16], buf4[16];
 
@@ -548,7 +601,7 @@ emit_preamble_pst (FILE * f, BOX_3D * ext, GLOBAL_ENV * env)
 }
 
 void
-emit_preamble_tkz (FILE * f, BOX_3D * ext, GLOBAL_ENV * env)
+emit_preamble_tkz_latex (FILE * f, BOX_3D * ext, GLOBAL_ENV * env)
 {
   char buf1[16], buf2[16], buf3[16], buf4[16];
   int picture_opts_p = 0;
@@ -590,15 +643,101 @@ emit_preamble_tkz (FILE * f, BOX_3D * ext, GLOBAL_ENV * env)
     }
 }
 
+// -----------------------------------------------------------------
+
+void
+emit_preamble_pst_context (FILE * f, BOX_3D * ext, GLOBAL_ENV * env)
+{
+  char buf1[16], buf2[16], buf3[16], buf4[16];
+
+  if (global_env_is_set_p (env, GE_OPTS))
+    {
+      fprintf (f, "\\psset{");
+      emit_opts_raw (f, env->opts, global_env->output_language);
+      fprintf (f, "}\n");
+    }
+
+  fprintf (f, 
+	   "%% ConTeXt does not yet support PSTricks.\n"
+	   "%% This is a guess at what the syntax might be.\n");
+
+  if (global_env_is_set_p (env, GE_FRAME))
+    {
+      if (env->frame_opts)
+	fprintf (f, "\\psframebox[%s]{", env->frame_opts);
+      else
+	fprintf (f, "\\psframebox[framesep=0pt]{");
+    }
+
+  fprintf (f, "\\startpspicture%s",
+	   global_env_is_set_p (env, GE_EXTENT) ? "*" : "");
+
+  if (global_env_is_set_p (env, GE_BASELINE))
+    fprintf (f, "[%s]", flt_str (buf1, env->baseline));
+
+  fprintf (f,
+	   "(%s,%s)(%s,%s)\n",
+	   flt_str (buf1, ext->min[X]),
+	   flt_str (buf2, ext->min[Y]),
+	   flt_str (buf3, ext->max[X]), flt_str (buf4, ext->max[Y]));
+
+  fprintf (f, "\\pstVerb{1 setlinejoin}\n");
+}
+
+void
+emit_preamble_tkz_context (FILE * f, BOX_3D * ext, GLOBAL_ENV * env)
+{
+  char buf1[16], buf2[16], buf3[16], buf4[16];
+  int picture_opts_p = 0;
+
+  if (global_env_is_set_p (env, GE_FRAME))
+    {
+      if (env->frame_opts)
+	warn (no_line, "frame options [%s] ignored (TikZ)", env->frame_opts);
+      else
+	{
+	  fprintf (f, "{\\fboxsep=0pt\\fbox{");
+	  warn (no_line,
+		"remove frame around TikZ/PGF pictures for debugging");
+	}
+    }
+
+  fprintf (f, "\\starttikzpicture[join=round");
+  if (global_env_is_set_p (env, GE_OPTS))
+    {
+      fprintf (f, ",");
+      emit_opts_raw (f, env->opts, global_env->output_language);
+    }
+  if (global_env_is_set_p (env, GE_BASELINE))
+    {
+      fprintf (f, ",");
+      fprintf (f, "baseline=%s", flt_str (buf1, env->baseline));
+    }
+  fprintf (f, "]\n");
+  if (global_env_is_set_p (env, GE_EXTENT))
+    {
+      flt_str (buf1, ext->min[X]);
+      flt_str (buf2, ext->min[Y]);
+      flt_str (buf3, ext->max[X]);
+      flt_str (buf4, ext->max[Y]);
+      fprintf (f,
+	       "\\useasboundingbox(%s,%s) rectangle (%s,%s);\n"
+	       "\\clip(%s,%s) rectangle (%s,%s);\n",
+	       buf1, buf2, buf3, buf4, buf1, buf2, buf3, buf4);
+    }
+}
+
 typedef void (*EMIT_PREAMBLE_FUNC) (FILE * f, BOX_3D * ext, GLOBAL_ENV * env);
 
 EMIT_PREAMBLE_FUNC emit_preamble_tbl[] = {
-  emit_preamble_pst,
-  emit_preamble_tkz,
+  emit_preamble_pst_latex,
+  emit_preamble_tkz_latex,
+  emit_preamble_pst_context,
+  emit_preamble_tkz_context,
 };
 
 void
-emit_postamble_pst (FILE * f, GLOBAL_ENV * env)
+emit_postamble_pst_latex (FILE * f, GLOBAL_ENV * env)
 {
   fprintf (f, "\\end{pspicture%s}",
 	   global_env_is_set_p (env, GE_EXTENT) ? "*" : "");
@@ -607,18 +746,37 @@ emit_postamble_pst (FILE * f, GLOBAL_ENV * env)
 }
 
 void
-emit_postamble_tkz (FILE * f, GLOBAL_ENV * env)
+emit_postamble_tkz_latex (FILE * f, GLOBAL_ENV * env)
 {
   fprintf (f, "\\end{tikzpicture}");
   if (global_env_is_set_p (env, GE_FRAME))
     fprintf (f, "}}");
 }
 
+void
+emit_postamble_pst_context (FILE * f, GLOBAL_ENV * env)
+{
+  fprintf (f, "\\stoppspicture%s}",
+	   global_env_is_set_p (env, GE_EXTENT) ? "*" : "");
+  if (global_env_is_set_p (env, GE_FRAME))
+    fprintf (f, "}");
+}
+
+void
+emit_postamble_tkz_context (FILE * f, GLOBAL_ENV * env)
+{
+  fprintf (f, "\\stoptikzpicture");
+  if (global_env_is_set_p (env, GE_FRAME))
+    fprintf (f, "}}");
+}
+
 typedef void (*EMIT_POSTAMBLE_FUNC) (FILE * f, GLOBAL_ENV * env);
 
 EMIT_POSTAMBLE_FUNC emit_postamble_tbl[] = {
-  emit_postamble_pst,
-  emit_postamble_tkz,
+  emit_postamble_pst_latex,
+  emit_postamble_tkz_latex,  
+  emit_postamble_pst_context,
+  emit_postamble_tkz_context,
 };
 
 void
